
__all__ = ['LnavCommandLexer']

import re

from pygments.token import Whitespace, Text, Keyword, Literal
from pygments.lexers._mapping import LEXERS
from pygments.lexers.python import RegexLexer

class LnavCommandLexer(RegexLexer):
    name = 'lnav'

    flags = re.IGNORECASE
    tokens = {
        'root': [
            (r'\s+', Whitespace),
            (r':[\w\-]+', Keyword),
            (r'\<[\w\-]+\>', Literal.String.Doc),
            (r'.', Text),
        ]
    }

def setup(app):
    LEXERS['LnavCommandLexer'] = (
        '_ext.lnavlexer', 'lnav', ('lnav',), ('*.lnav',), ('text/lnav',))
    app.add_lexer('lnav', LnavCommandLexer)



#! /usr/bin/env python3

import os
import sys
import time
import uuid
import shutil
import random
import datetime

SYSLOG_DATE_FMT = "%b %d %H:%M:%S"
ACCESS_LOG_DATE_FMT = "%d/%b/%Y:%H:%M:%S"
GENERIC_DATE_FMT = "%Y-%m-%dT%H:%M:%S.%%s"

TEST_ADDRESSES = (
        ["192.0.2.55"] * 20 +
        ["192.0.2.44"] * 20 +
        ["192.0.2.3"] * 40 +
        ["192.0.2.100"] * 10 +
        ["192.0.2.122"] * 30 +
        ["192.0.2.42"] * 100
)

TEST_USERNAMES = [
    "bob@example.com",
    "bob@example.com",
    "bob@example.com",
    "combatcarl@example.com",
    "combatcarl@example.com",
    "combatcarl@example.com",
    "combatcarl@example.com",
    "combatcarl@example.com",
    "combatcarl@example.com",
    "-",
    "-",
    "-",
    "-",
    "-",
    "-",
]

TEST_METHODS = [
    "GET",
    "GET",
    "GET",
    "GET",
    "GET",
    "GET",
    "PUT",
]

PATH_COMPS = "Lorem ipsum dolor sit amet consectetur adipiscing elit sed do eiusmod tempor incididunt ut labore et dolore magna aliqua".split()

TEST_URLS = [
    "/index.html",
    "/index.html",
    "/index.html",
    "/features.html",
    "/images/compass.jpg",
    "/obj/1234",
    "/obj/1235?foo=bar",
    "/obj/1236?search=demo&start=1",
]

for index in range(0, 50):
    path_list = []
    for count in range(random.randint(2, 8)):
        path_list.append(random.choice(PATH_COMPS))
    TEST_URLS.append("/".join(path_list))

TEST_VERSIONS = [
    "HTTP/1.0",
    "HTTP/1.0",
    "HTTP/1.1",
]

TEST_STATUS = [
    200,
    200,
    200,
    200,
    200,
    200,
    200,
    200,
    200,
    200,
    404,
    404,
    404,
    404,
    403,
    403,
    403,
    500
]

TEST_REFERRERS = [
    "-",
    "-",
    "-",
    "-",
    "-",
    "-",
    "http://lnav.org/download.html",
]

TEST_AGENTS = [
    "-",
    "-",
    "-",
    "-",
    "Apache-HttpClient/4.2.3 (java 1.5)",
    "Apache-HttpClient/4.2.3 (java 1.5)",
    "Apache-HttpClient/4.2.3 (java 1.5)",
    "Apache-HttpClient/4.2.3 (java 1.5)",
    "Apache-HttpClient/4.2.3 (java 1.5)",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    "Roku4640X/DVP-7.70 (297.70E04154A)",
    "Roku4640X/DVP-7.70 (297.70E04154A)",
    "Roku4640X/DVP-7.70 (297.70E04154A)",
]

START_TIME = datetime.datetime.fromtimestamp(1641898727)
ACCESS_LOG_CURR_TIME = START_TIME
SYSLOG_LOG_CURR_TIME = START_TIME


def access_log_msgs():
    global ACCESS_LOG_CURR_TIME
    while True:
        ACCESS_LOG_CURR_TIME += datetime.timedelta(seconds=random.randrange(1, 3))
        yield '%s - %s [%s +0000] "%s %s %s" %s %s "%s" "%s"\n' % (
            random.choice(TEST_ADDRESSES),
            random.choice(TEST_USERNAMES),
            ACCESS_LOG_CURR_TIME.strftime(ACCESS_LOG_DATE_FMT),
            random.choice(TEST_METHODS),
            random.choice(TEST_URLS),
            random.choice(TEST_VERSIONS),
            random.choice(TEST_STATUS),
            int(random.lognormvariate(1, 1) * 1000),
            random.choice(TEST_REFERRERS),
            random.choice(TEST_AGENTS)
        )


TEST_PROCS = [
    "server[123]",
    "server[123]",
    "server[123]",
    "server[121]",
    "server[124]",
    "server[123]",
    "worker[61456]",
    "worker[61456]",
    "worker[61457]",
]

TEST_MSGS = [
    "Handling request %s" % uuid.uuid4(),
    "Handling request %s" % uuid.uuid4(),
    "Handling request %s" % uuid.uuid4(),
    "Successfully started helper",
    "Reading from device: /dev/hda",
    "Received packet from %s" % random.choice(TEST_ADDRESSES),
    "Received packet from %s" % random.choice(TEST_ADDRESSES),
    "Received packet from %s" % random.choice(TEST_ADDRESSES),
]


def syslog_msgs():
    global SYSLOG_LOG_CURR_TIME
    while True:
        SYSLOG_LOG_CURR_TIME += datetime.timedelta(seconds=random.randrange(1, 3))
        yield '%s frontend3 %s: %s\n' % (
            SYSLOG_LOG_CURR_TIME.strftime(SYSLOG_DATE_FMT),
            random.choice(TEST_PROCS),
            random.choice(TEST_MSGS),
        )


try:
    shutil.rmtree("/tmp/demo")
    os.makedirs("/tmp/demo")
except OSError:
    pass

FILES = [
    ("/tmp/demo/access_log", access_log_msgs()),
    ("/tmp/demo/messages", syslog_msgs()),
]

COUNTER = 0
while COUNTER < 5000:
    loop_inc = datetime.timedelta(seconds=random.weibullvariate(1, 1.5) * 500)
    ACCESS_LOG_CURR_TIME += loop_inc
    SYSLOG_LOG_CURR_TIME += loop_inc
    for fname, gen in FILES:
        for i in range(random.randrange(4, 8)):
            COUNTER += 1
            with open(fname, "a+") as fp:
                if random.uniform(0.0, 1.0) < 0.01:
                    line = next(gen)
                    prefix = line[:50]
                    suffix = line[50:]
                    fp.write(prefix)
                    # time.sleep(random.uniform(0.5, 0.6))
                    fp.write(suffix)
                else:
                    fp.write(next(gen))
                # if random.uniform(0.0, 1.0) < 0.010:
                #    fp.truncate(0)
            time.sleep(random.uniform(0.01, 0.02))
            # if random.uniform(0.0, 1.0) < 0.001:
            #    os.remove(fname)



#! /usr/bin/env python

# Copyright (c) 2013, Timothy Stack
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# * Neither the name of Timothy Stack nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import string
import readline
import itertools
import collections

TEST_DIR = os.path.dirname(__file__)
ROOT_DIR = os.path.dirname(TEST_DIR)
SRC_DIR = os.path.join(ROOT_DIR, "src")

addr_to_name = {}
name_to_addr = {}
element_lists = collections.defaultdict(list)
list_depth = {}
list_format = {}
breakpoints = set()

def completer(text, state):
    options = [x for x in itertools.chain(name_to_addr,
                                          element_lists,
                                          breakpoints)
               if x.startswith(text)]
    try:
        return options[state]
    except IndexError:
        return None

readline.set_completer(completer)

if 'libedit' in readline.__doc__:
    readline.parse_and_bind('bind ^I rl_complete')
else:
    readline.parse_and_bind('tab: complete')

input_line = ''
ops = []
for line in open("scanned.dpt"):
    if line.startswith("input "):
        input_line = line[6:-1]
    else:
        ops.append(map(string.strip, line.split()))

def getstr(capture):
    start, end = capture.split(':')
    return input_line[int(start):int(end)]

def printlist(name_or_addr):
    if name_or_addr in name_to_addr:
        addr = name_to_addr[name_or_addr]
        print "% 3d (%s:%s) %s" % (list_depth.get(addr, -1), name_or_addr, addr, element_lists[addr])
    elif name_or_addr in element_lists:
        addr = name_or_addr
        print "% 3d (%s:%s) %s" % (list_depth.get(name_or_addr, -1),
                                addr_to_name.get(name_or_addr, name_or_addr),
                                name_or_addr,
                                element_lists[name_or_addr])
    else:
        print "error: unknown list --", name_or_addr

    if addr in list_format:
        print "    format -- appender(%s) term(%s) qual(%s) sep(%s) prefix_term(%s)" % tuple(list_format[addr])

def handleop(fields):
    addr = fields[0]
    loc = fields[1].split(':')
    method_name = fields[2]
    method_args = fields[3:]

    if addr == '0x0':
        el = None
    else:
        el = element_lists[addr]

    if method_name == 'element_list_t':
        addr_to_name[addr] = method_args[0]
        name_to_addr[method_args[0]] = addr
        list_depth[addr] = int(method_args[1])
    elif method_name == '~element_list_t':
        del element_lists[addr]
    elif method_name == 'format':
        list_depth[addr] = int(method_args[0])
        list_format[addr] = method_args[1:]
    elif method_name == 'consumed':
        list_depth[addr] = -1
    elif method_name == 'push_back':
        el.append((method_args[0], getstr(method_args[1])))
    elif method_name == 'pop_front':
        el.pop(0)
    elif method_name == 'pop_back':
        el.pop()
    elif method_name == 'clear2':
        el[::] = []
    elif method_name == 'splice':
        pos = int(method_args[0])
        other = element_lists[method_args[1]]
        start, from_end = map(int, method_args[2].split(':'))
        end = len(other) - from_end
        sub_list = other[start:end]
        del other[start:end]
        el[pos:pos] = sub_list
    elif method_name == 'swap':
        other = element_lists[method_args[0]]
        element_lists[method_args[0]] = el
        element_lists[addr] = other
    elif method_name == 'point':
        breakpoints.add(method_args[0])
    else:
        print "Unhandled method: ", method_name

def playupto(length):
    addr_to_name.clear()
    name_to_addr.clear()
    element_lists.clear()
    list_depth.clear()
    for index in range(length):
        handleop(ops[index])

def find_prev_point(start, name):
    orig_start = start
    while start > 0:
        start -= 1;
        fields = ops[start]
        if fields[2] != 'point':
            continue
        if not name or fields[3] == name:
            return start + 1
    return orig_start + 1

def find_next_point(start, name):
    orig_start = start
    while start < len(ops):
        start += 1;
        fields = ops[start]
        if fields[2] != 'point':
            continue
        if not name or fields[3] == name:
            return start + 1
    return orig_start + 1

def printall():
    print input_line
    sorted_lists = [(list_depth.get(addr, -1), addr) for addr in element_lists]
    sorted_lists.sort()
    for _depth, addr in sorted_lists:
        printlist(addr)

index = len(ops)
last_cmd = ['']
watch_list = set()
while True:
    playupto(index)

    if index == 0:
        print "init"
    else:
        op = ops[index - 1]
        print "#%s %s" % (index -1, op)
        if op[2] == 'push_back':
            print getstr(op[4])

    for list_name in watch_list:
        printlist(list_name)

    try:
        cmd = raw_input("> ").split()
    except EOFError:
        print
        break

    if not cmd or cmd[0] == '':
        cmd = last_cmd

    if not cmd or cmd[0] == '':
        pass
    elif cmd[0] == 'h':
        print 'Help:'
        print '  q - quit'
        print '  s - Start over'
        print '  n - Next step'
        print '  r - Previous step'
        print '  b - Previous breakpoint'
        print '  c - Next breakpoint'
        print '  p - Print state'
        print '  w <var> - Add a variable to the watch list'
        print '  u <var> - Remove a variable from the watch list'
    elif cmd[0] == 'q':
        break
    elif cmd[0] == 's':
        index = 0
    elif cmd[0] == 'n':
        if index < len(ops):
            index += 1
    elif cmd[0] == 'r':
        if index > 0:
            index -= 1
    elif cmd[0] == 'b':
        if len(cmd) == 1:
            cmd.append('')

        index = find_prev_point(index - 1, cmd[1])
    elif cmd[0] == 'c':
        if len(cmd) == 1:
            cmd.append('')
        index = find_next_point(index - 1, cmd[1])
    elif cmd[0] == 'p':
        if len(cmd) > 1:
            printlist(cmd[1])
        else:
            printall()
    elif cmd[0] == 'w':
        watch_list.add(cmd[1])
    elif cmd[0] == 'u':
        if watch_list:
            watch_list.remove(cmd[1])
    else:
        print "error: unknown command --", cmd

    printall()

    last_cmd = cmd



from conans import ConanFile
from conan.tools.cmake import CMake, CMakeToolchain, CMakeDeps


class LnavConan(ConanFile):
    name = "lnav"
    version = "0.11.2"
    homepage = "https://lnav.org"
    url = "https://github.com/tstack/lnav.git"
    license = "BSD-2-Clause"
    description = (
        "The Log File Navigator, lnav for short, is an advanced "
        "log file viewer for the small-scale"
    )
    settings = "os", "compiler", "build_type", "arch"
    exports_sources = "*"
    no_copy_source = True
    requires = (
        "bzip2/1.0.8",
        "libarchive/3.6.0",
        "libcurl/7.85.0",
        "ncurses/6.3",
        "pcre2/10.40",
        "readline/8.1.2",
        "sqlite3/3.38.0",
        "zlib/1.2.12",
    )
    generators = ("virtualrunenv",)
    default_options = {
        "libarchive:with_bzip2": True,
        "libarchive:with_lz4": True,
        "libarchive:with_lzo": True,
        "libarchive:with_lzma": True,
        "libarchive:with_zstd": True,
        "pcre2:support_jit": True,
        "pcre2:build_pcre2_8": True,
        "sqlite3:enable_json1": True,
        "sqlite3:enable_soundex": True,
        "readline:with_library": "curses",
    }

    def generate(self):
        CMakeToolchain(self).generate()
        CMakeDeps(self).generate()

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        if self.settings.os == "Macos" and self.settings.arch == "armv8":
            cmake.definitions["CMAKE_SYSTEM_PROCESSOR"] = "arm64"
        cmake.build()

    def package(self):
        cmake = CMake(self)
        cmake.install()

    def deploy(self):
        self.copy("*", dst="bin", src="bin")
